@@ -323,47 +323,6 @@ done:
       retval = PAM_SUCCESS;
     }
   DBG (("done. [%s]", pam_strerror (pamh, retval)));
-  pam_set_data (pamh, "oath_setcred_return", (void*) (intptr_t) retval, NULL);
-
-  return retval;
-}
-
-PAM_EXTERN int
-pam_sm_setcred (pam_handle_t * pamh, int flags, int argc, const char **argv)
-{
-  int retval;
-  int auth_retval;
-  struct cfg cfg;
-
-  parse_cfg (flags, argc, argv, &cfg);
-
-  DBG (("called."));
-
-  /* TODO: ? */
-
-  retval = pam_get_data (pamh, "oath_setcred_return",
-			 (void*) (intptr_t) &auth_retval);
-  DBG (("retval: %d", auth_retval));
-  if (retval != PAM_SUCCESS)
-    return PAM_CRED_UNAVAIL;
-
-  switch (auth_retval)
-    {
-    case PAM_SUCCESS:
-      retval = PAM_SUCCESS;
-      break;
-
-    case PAM_USER_UNKNOWN:
-      retval = PAM_USER_UNKNOWN;
-      break;
-
-    case PAM_AUTH_ERR:
-    default:
-      retval = PAM_CRED_ERR;
-      break;
-    }
-
-  DBG (("done. [%s]", pam_strerror (pamh, retval)));
 
   return retval;
 }
@@ -373,7 +332,7 @@ pam_sm_setcred (pam_handle_t * pamh, int flags, int argc, const char **argv)
 struct pam_module _pam_oath_modstruct = {
   "pam_oath",
   pam_sm_authenticate,
-  pam_sm_setcred,
+  NULL,
   NULL,
   NULL,
   NULL,
